
#if defined(linux) || (defined (__FreeBSD__) && defined(i386)) || defined(__NetBSD__)

#include "ywindow.h"
#include "ytimer.h"
#include "ymsgbox.h"


#if defined(__FreeBSD__) || defined(__NetBSD__) || defined(__OpenBSD__)
#define APMDEV "/dev/apm"
#else
#define APMDEV "/proc/apm"
#endif

//assume there is no laptop with more
//than 3 batteries
#define MAX_ACPI_BATTERY_NUM 3

typedef struct {
  //(file)name of battery
  int present;
  char *name;
  int capacity_full;
} bat_info;
  

class YApm: public YWindow, public YTimerListener, public YActionListener, public YMsgBoxListener {
public:
    YApm(YWindow *aParent = 0);
    virtual ~YApm();

    virtual void paint(Graphics &g, const YRect &r);

    void updateToolTip();
    virtual bool handleTimer(YTimer *t);
    void msgBox(char *message);
    void msgBoxClose();
    void notifyLowBattery();
    void notifyAC_on();
    void notifyAC_off();
    virtual void handleMsgBox(YMsgBox *, int);
    virtual void handleButton(const XButtonEvent &button);
    virtual void actionPerformed(YAction*, unsigned int);

private:
    YTimer *apmTimer;

    ref<YPixmap> getPixmap(char ch);
    int calcInitialWidth();
    int calcWidth(const char *s, int count);

    void ApmStr(char *s, bool Tool);
    void AcpiStr(char *s, bool Tool);
    void PmuStr(char *, const bool);
    int ignore_directory_bat_entry(struct dirent *de);


    float percent;
    int   rtime;
    int   plugged;
    int   charging;
    int   notified;
    YMsgBox *fApmDialog;

    static YColor *apmBg;
    static YColor *apmFg;
    static YColor *apmFull;
    static YColor *apmEmpty;
    static YColor *apmBorder;
    static ref<YFont> apmFont;

    // display mode: pmu, acpi or apm info
    enum { APM, ACPI, PMU } mode;
    //number of batteries (for apm == 1)
    int batteryNum;
    //names of batteries to ignore. e.g.
    //the laptop has two slots but the
    //user has only one battery
    static char *acpiIgnoreBatteryNames;
    //list of batteries (static info)
    bat_info *acpiBatteries[MAX_ACPI_BATTERY_NUM];
    //(file)name of ac adapter
    char *acpiACName;
    char *fCurrentState;
    char *fCurrentStateTool;

    void updateState();
};
#else
#undef CONFIG_APPLET_APM
#endif
