/*
 * IceWM
 *
 * Copyright (C) 2004 Igor Stirbu
 *
 * Derived from aapm by Andreas Hinz and aclock by Marko Macek
 *
 * Sound mixer
 */

#define NEED_TIME_H


#include "config.h"
#include "amixer.h"

#ifdef CONFIG_APPLET_MIXER

#include "ylib.h"
#include "sysdep.h"

#include "aclock.h"

#include "yapp.h"
#include "prefs.h"
#include "intl.h"

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

#include <sys/types.h>
#include <sys/ioctl.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <stdio.h>
#include <linux/soundcard.h>
#include <errno.h>



static YColor *taskBarBg = 0;

extern ref<YPixmap> traybackPixmap;


#ifndef CONFIG_TOOLTIP
void MYWindow::handleCrossing(const XCrossingEvent &/*crossing*/) {
#else
void MYWindow::handleCrossing(const XCrossingEvent &crossing) {
    if (fToolTip) {
        if (fToolTipTimer && crossing.type == MotionNotify) { //MCMMIX
            fToolTipTimer->setTimerListener(fToolTip); //MCMMIX
            fToolTipTimer->runTimer(); //MCMMIX
        } else if (crossing.type == EnterNotify && crossing.mode == NotifyNormal) {
            if (fToolTipTimer == 0)
                fToolTipTimer = new YTimer(1);
            if (fToolTipTimer) {
                fToolTipTimer->setTimerListener(fToolTip);
                fToolTipTimer->startTimer();
                updateToolTip();
                if (fToolTip)
                    fToolTip->locate(this, crossing);
            }
        } else if (crossing.type == LeaveNotify) {
            fToolTip->hide();
            if (fToolTipTimer && fToolTipTimer->getTimerListener() == fToolTip) {
                fToolTipTimer->stopTimer();
                fToolTipTimer->setTimerListener(0);
            }
        }
    }
#endif
}

#ifndef CONFIG_TOOLTIP
void MYWindow::setToolTip(const char */*tip*/) {
#else
YTimer *MYWindow::fToolTipTimer = 0;
void MYWindow::setToolTip(const char *tip) {
    if (fToolTip) {
        if (!tip) {
            delete fToolTip; fToolTip = 0;
        } else {
            fToolTip->setText(tip);
            fToolTip->repaint();
        }
    }
    if (tip) {
        if (!fToolTip)
            fToolTip = new YToolTip();
        if (fToolTip)
            fToolTip->setText(tip);
    }
#endif
}



YMixer::YMixer(YWindow *aParent): MYWindow(aParent) {

	if (taskBarBg == 0) {
		taskBarBg = new YColor(clrDefaultTaskBar);
	}

#if USE_PIXMAP
	init_pixmaps();
#endif
	mixer_init();

	if (mixer_fd < 0) {
		setSize(0,0);
		return;
	}

	last_val = 100;

	mixerTimer = new YTimer(500);
	mixerTimer->setTimerListener(this);
	mixerTimer->startTimer();

	setSize(20, taskButtonHeight);

	updateToolTip();
}

#if USE_PIXMAP
int YMixer::init_pixmaps()
{
	YResourcePaths const paths;
	char const *base("mixer/");
	YResourcePaths subdirs(paths, base);
	subdirs.init(paths, base);

	PixMute = subdirs.loadPixmap(base, "volume-mute.xpm");
	PixMix[0] = subdirs.loadPixmap(base, "volume-0.xpm");
	PixMix[1] = subdirs.loadPixmap(base, "volume-1.xpm");
	PixMix[2] = subdirs.loadPixmap(base, "volume-2.xpm");
	PixMix[3] = subdirs.loadPixmap(base, "volume-3.xpm");
	PixMix[4] = subdirs.loadPixmap(base, "volume-4.xpm");

	return 0;
}
#endif

int YMixer::mixer_write(int height)
{
	int value;

	if (mixer_fd < 0) return -1;

	value = height | (height << 8);

	if (ioctl(mixer_fd, SOUND_MIXER_WRITE_VOLUME, &value) == -1) {
		return -1;
	}
	//if (height)
		//muted_state=height;
	return 0;

}

int YMixer::mixer_read()
{
	int value=10;

	if (mixer_fd < 0) return -1;

	if (ioctl(mixer_fd, SOUND_MIXER_READ_VOLUME, &value) == -1) {
		return (0);
	}

	return (value & 0xFF);
}

int YMixer::mixer_init() 
{
	if ( access(mixerDEV, W_OK | R_OK ) != 0  || (mixer_fd = open(mixerDEV, O_RDWR, 0)) < 0) {
		mixer_fd = -1;
	} else {
		fcntl(mixer_fd, F_SETFD, FD_CLOEXEC);
	}
	return 0;
}
int YMixer::mixer_ismuted()
{
	if (mixer_fd < 0) return 0;
	if (mixer_read() <= 0)
		return 1;
	return 0;
}
int YMixer::mixer_mute()
{
	if (mixer_fd < 0) return -1;
	last_val = mixer_read();
	return mixer_write(0);
}

int YMixer::mixer_unmute()
{
	if (mixer_fd < 0) return -1;
	int mute_val=1; 
	ioctl(mixer_fd, SOUND_MIXER_WRITE_MUTE, &mute_val);
	return mixer_write(last_val <= 0 ? 1 : last_val);
}
void YMixer::updateMute() {
	int muted = mixer_ismuted();
	if (muted)
		mixer_unmute();
	else
		mixer_mute();
}

void YMixer::handleButton(const XButtonEvent &button) {
	YWindow::handleButton(button);
	if (button.type == ButtonPress) {
		int val = mixer_read(); 
		int muted = mixer_ismuted();
		int b=button.button;
		if (muted) {
			mixer_unmute();
		} else {
			if (b == 2) {
				mixer_mute();
			} else if ( (b==4 || b==1) && val <= 95) {
				val +=5;
				val -= val%5;
				mixer_write(val);
			} else if ( (b==5 || b==3) && val >= 5 ) {
				val -=5;
				val -= val%5;
				mixer_write(val);
			}
		}
		updateToolTip();
		repaint();
		XCrossingEvent crossing; //MCM
		crossing.type = MotionNotify;
		handleCrossing(crossing);
	}
}

YMixer::~YMixer() {
	if (mixer_fd >= 0)
	      close(mixer_fd);
}

void YMixer::updateToolTip() {
	if (mixer_fd < 0) return;
        char s[100]; *s=0;
	int val = mixer_read();
	int muted = mixer_ismuted();
        if (muted) {
		val = last_val;
                sprintf(s, "%s %s\n", _("Volume:"),_("Muted"));
        } else {
                sprintf(s, "%s %d%%", _("Volume:"),val);
		strcat(s,val>99? "\n[" :"  \n[");
		for (int i=0;i<100;i+=5)
                	strcat(s, (i<val?"|":"·"));
		strcat(s,"]");
        }
        setToolTip(s);
}


void YMixer::paint(Graphics &g, const YRect &/*r*/) {
	if (mixer_fd < 0) return;

	int val = mixer_read();
	int muted = mixer_ismuted();
	//if (last_val == val && last_muted == muted)
		//return;

        if (traybackPixmap != null)
             g.fillPixmap(traybackPixmap, 0, 0,
                         width(), height(), this->x(), this->y());
        else {
	     g.setColor(taskBarBg);
	     g.fillRect(0, 0, width(), height());
        }
#if USE_PIXMAP 
	int i = (muted ? last_val : val) * PIX_NUM / 100;
	i -= i==PIX_NUM? 1 : 0;
	i = i >= PIX_NUM ? 0 : i;

	ref<YPixmap> p = PixMix[i];
        if (p != null )
	    g.drawPixmap(p, (width() - p->width()) / 2, (height() - p->height()) / 2);
	if (val > 0)
		last_val = val;
	last_muted = muted;
	p = PixMute;
        if (muted && p != null )
	    g.drawPixmap(p, (width() - p->width()) / 2, (height() - p->height()) / 2);
#endif

}

bool YMixer::handleTimer(YTimer *t) {
	if (t != mixerTimer) return false;

	if (toolTipVisible()) {
		updateToolTip();
	}
	        repaint();
	return true;
}
#endif
