/*
 * IceWM
 *
 * Copyright (C) 1998-2003 Marko Macek & Nehal Mistry
 *
 * Changes:
 *
 *      2003/06/14
 *       * created gnome2 support from gnome.cc
 */

#include "config.h"
#ifdef CONFIG_GNOME_MENUS

#include "ylib.h"
#include "default.h"

#include "ypixbuf.h"
#include "yapp.h"
#include "sysdep.h"
#include "base.h"
#include <dirent.h>
#include <string.h>

#include <gnome.h>
#include <libgnome/gnome-desktop-item.h>
#include <libgnomevfs/gnome-vfs-init.h>
#include "yarray.h"

char const *ApplicationName("icew-menu-gnome2");

#define MAX_APP_ITEMS 999
int showEmptyMenu = 0;
#define APPLICATIONS_DIR "/usr/share/applications/"
#define DIRECTORIES_DIR  "/usr/share/desktop-directories/"

typedef struct {
   const char *filename;
   const char *includedCategories[10];
   const char *excludedCategories[10];
   const char *subMenus[10];
   bool  isSubmenu;
} MenuDir;

// Configuration for the root menu
MenuDir  root=           {0,            {"PackageManager",0},
                                        {0}, 
                                        {0},
                                        false};

// Known configuration menus
MenuDir  knownMenuDirs[]= {
   {"Accessories.directory",            {"Utility",0}, 
                                        {"System",0}, 
                                        {0},
                                        false},
   {"Development.directory",            {"Development",0}, 
                                        {0}, 
                                        {0},
                                        false},
   {"Edutainment.directory",            {"Education",0}, 
                                        {0}, 
                                        {0},
                                        false},
   {"Games.directory",                  {"Game",0}, 
                                        {0}, 
                                        {0},
                                        false},
   {"Graphics.directory",               {"Graphics",0}, 
                                        {0}, 
                                        {0},
                                        false},
   {"Internet.directory",               {"Network",0}, 
                                        {0}, 
                                        {0},
                                        false},
   {"Network.directory",                {"Network",0}, 
                                        {0}, 
                                        {0},
                                        false},
   {"Multimedia.directory",             {"AudioVideo",0}, 
                                        {"System",0}, 
                                        {0},
                                        false},
   {"Office.directory",                 {"Office",0}, 
                                        {0}, 
                                        {0},
                                        false},
   {"System.directory",                 {"System",0}, 
                                        {"Screensaver","SystemSetup","Monitor","X-Red-Hat-ServerConfig",0}, 
                                        {0},
                                        false},
   {"System-Tools.directory",           {"System",0}, 
                                        {"Settings",0}, 
                                        {0},
                                        false},
   {"SystemConfig.directory",           {"Application","SystemSetup","X-Red-Hat-Base",0}, 
                                        {"X-Red-Hat-ServerConfig","Screensaver",0}, 
                                        {"ServerConfig.directory",0},
                                        false},
   {"ServerConfig.directory",           {"Application","X-Red-Hat-ServerConfig","X-Red-Hat-Base", }, 
                                        {0}, 
                                        {0},
                                        true},
   {"Settings.directory",               {"Settings","X-Red-Hat-Base",0}, 
                                        {"System","Accessibility",0}, 
                                        {"Settings-Accessibility.directory","Settings-More.directory",0},
                                        false},
   {"Preferences.directory",            {"Settings",0}, 
                                        {"System","Accessibility",0}, 
                                        {"Settings-Accessibility.directory","Settings-More.directory",0},
                                        false},
   {"Settings-Accessibility.directory", {"Settings","Accessibility",0}, 
                                        {"System",0}, 
                                        {0},
                                        true},
   {"Settings-More.directory",          {"Settings",0}, 
                                        {"System","X-Red-Hat-Base-Only","X-Red-Hat-Base",0}, 
                                        {0},
                                        true},

   {0, {0}, {0}, {0},false}
};

class GnomeMenu;

class GnomeMenuItem {
public:
    GnomeMenuItem() { title = 0; icon = 0; dentry = 0; command=0; ditem=0; submenu = 0; }

    const char *title;
    const char *icon;
    const char *dentry;
    const char *command;
    GnomeDesktopItem *ditem;
    GnomeMenu *submenu;
};

GnomeDesktopItem *cacheAppItems[MAX_APP_ITEMS];
GnomeDesktopItem *cacheItems[MAX_APP_ITEMS];

int itemComparator(GnomeDesktopItem **item1, GnomeDesktopItem **item2) {
    if (item1 && *item1 && item2 && *item2 ) {
        const char *name1 = gnome_desktop_item_get_localestring(*item1, GNOME_DESKTOP_ITEM_NAME);
        const char *name2 = gnome_desktop_item_get_localestring(*item2, GNOME_DESKTOP_ITEM_NAME);
        if (name1 && name2)
            return(strcasecmp(name1,name2));
    }
    return(0);
}

void SortItems(GnomeDesktopItem **items) {
  int cont=0;
  for (cont=0; items[cont]; cont++);
  qsort(items,cont,sizeof(GnomeDesktopItem *),(int (*) (const void *, const void *))itemComparator);
}


class GnomeMenu {
public:
    GnomeMenu() { }

    YObjectArray<GnomeMenuItem> items;

    bool isDuplicateName(const char *name);
    void addEntry(const char *fPath, const char *name, const int plen,
                         GnomeDesktopItem *ditem, const bool firstRun);
    void populateMenu(const char *fPath);
};

void dumpMenu(GnomeMenu *menu, int ind) {
    for (unsigned int i = 0; i < menu->items.getCount(); i++) {
        GnomeMenuItem *item = menu->items.getItem(i);

        // print applications entries
        if (item && item->dentry && !item->submenu && item->command && *item->command) {
            const char *cmd = item->command;
            char *c= (char *)cmd+strlen(cmd)-1;
            // desktop items has parameters that starts with % (%u %f ...)
            for (;c>cmd;c--) if (*c=='%') *c=0;
            // delete final spaces
            c= (char *)cmd+strlen(cmd)-1;
            for (;c>cmd;c--) 
               if (*c==' ') *c=0;
               else break;
            // delete begin spaces
            while(*cmd == ' ') cmd++;

            // print indentation spaces
            for(int cc = 0; cc < ind; cc++) printf("   ");

            // print the program entry
            printf("prog \"%s\" %s %s\n",
                   item->title,
                   item->icon ? item->icon : "-",
                   item->command);

        // print submenus
        } else if (item && item->dentry && item->submenu) {
            GnomeMenu *submenu = item->submenu;
            if (showEmptyMenu || submenu->items.getCount()) {
                const char *c;
                c=strrchr(item->dentry, '/');
                if (!c) c= item->dentry;

                // print indentation spaces
                for(int cc = 0; cc < ind; cc++) printf("   ");
                // open the menu entry
                printf("menu \"%s\" \"%s\" {\n", item->title, item->icon ? item->icon : "-");

                // call to this function recurs.
                dumpMenu(submenu, ind + 1);

                // print indentation spaces
                for(int cc = 0; cc < ind; cc++) printf("   ");
                // close the menu entry
                printf("}\n");
    }
}
    }
}

void GnomeMenu::addEntry(const char *fPath, const char *name, const int plen,
                         GnomeDesktopItem *ditem, const bool firstRun)
{
    const int nlen = (plen == 0 || fPath[plen - 1] != '/')
        ? plen + 1 + strlen(name)
        : plen + strlen(name);
    char *npath = new char[nlen + 1];

    if (npath) {
        strcpy(npath, fPath);

        if (plen == 0 || npath[plen - 1] != '/') {
            npath[plen] = '/';
            strcpy(npath + plen + 1, name);
        } else
            strcpy(npath + plen, name);

        GnomeMenuItem *item = new GnomeMenuItem();
        item->title = name;

        if (ditem == NULL ) 
            ditem = gnome_desktop_item_new_from_file(npath,
                                             (GnomeDesktopItemLoadFlags)0,
                                             NULL);
        if (ditem == NULL)
            return;

        struct stat sb;
        const char *type;
        bool isDir = (!stat(npath, &sb) && S_ISDIR(sb.st_mode));
        type = gnome_desktop_item_get_string(ditem,
                                             GNOME_DESKTOP_ITEM_TYPE);
        if (!isDir && type && strstr(type, "Directory")) {
            isDir = 1;
        }

        if (isDir) {
            GnomeMenu *submenu = new GnomeMenu();
            submenu->populateMenu(npath);

            item->title = g_basename(npath);
            item->icon = gnome_pixmap_file("gnome-folder.png");
            item->submenu = submenu;

            item->title = gnome_desktop_item_get_localestring(ditem, GNOME_DESKTOP_ITEM_NAME);
                item->icon = gnome_desktop_item_get_string(ditem, GNOME_DESKTOP_ITEM_ICON);
            item->dentry = npath;
        } else {
            if (type && !strstr(type, "Directory")) {
                item->title = gnome_desktop_item_get_localestring(ditem, GNOME_DESKTOP_ITEM_NAME);
                item->command = gnome_desktop_item_get_string(ditem, GNOME_DESKTOP_ITEM_EXEC);
                if (gnome_desktop_item_get_string(ditem, GNOME_DESKTOP_ITEM_ICON))
                    item->icon = gnome_desktop_item_get_string(ditem, GNOME_DESKTOP_ITEM_ICON);
                item->dentry = npath;
            }
        }
        const char *no_display = gnome_desktop_item_get_localestring(ditem, GNOME_DESKTOP_ITEM_NO_DISPLAY);
        bool  displayIt = (no_display && *no_display == 'd') ? false : true;
        if (firstRun || displayIt) {
            items.append(item);
            gnome_desktop_item_set_string(ditem,GNOME_DESKTOP_ITEM_NO_DISPLAY,"done");
        }
    }
}

void GnomeMenu::populateMenu(const char *fPath) {
    struct stat sb;
    bool isDir = (!stat(fPath, &sb) && S_ISDIR(sb.st_mode));
    const int plen = strlen(fPath);

    char tmp[256];
    strcpy(tmp, fPath);
    strcat(tmp, "/.directory");

    if (isDir && !stat(tmp, &sb)) { // looks like kde/gnome1 style

        char *opath = new char[plen + sizeof("/.order")];
        if (opath) {
            strcpy(opath, fPath);
            strcpy(opath + plen, "/.order");

            FILE * order(fopen(opath, "r"));

            if (order) {
                char oentry[100];

                while (fgets(oentry, sizeof (oentry), order)) {
                    const int oend = strlen(oentry) - 1;

                    if (oend > 0 && oentry[oend] == '\n')
                        oentry[oend] = '\0';

                    addEntry(fPath, oentry, plen, NULL, true);
                }
                fclose(order);
            }
            delete opath;
        }

        struct dirent **namelist;
        int i, n;
        if ( 0 < (n = scandir(fPath, &namelist, 0, alphasort)) ) {
            for (i=0; i<n; i++) {
                if (*namelist[i]->d_name != '.') 
                    addEntry(fPath, namelist[i]->d_name, plen, NULL, false);
                free(namelist[i]);
            }
            free(namelist);
        }
    } else {   // gnome2 style

        char dirname[256] = "";
        if (isDir) 
            strcpy(dirname, fPath);

        // Read all .directories files in directory, and cache them (cacheItems)
        if (isDir) {
            DIR *dir = opendir(dirname);
            if (dir != 0) {
                int cont=0;
                struct dirent *file;
                while ((file = readdir(dir)) != NULL) {
                    if (*file->d_name == '.') 
                        continue;

                    // Only consider known filenames
                    bool ok = false;
                    for (int i=0; knownMenuDirs[i].filename && !ok ; i++) 
                        if (!strcmp(file->d_name, knownMenuDirs[i].filename) && !knownMenuDirs[i].isSubmenu) 
                             ok=true;
                    if (!ok)
                        continue;

                    char fullpath[256];
                    strcpy(fullpath, dirname);
                    strcat(fullpath, "/");
                    strcat(fullpath, file->d_name);
                    GnomeDesktopItem *ditem =
                             gnome_desktop_item_new_from_file(fullpath,
                                                             (GnomeDesktopItemLoadFlags)0,
                                                             NULL);
                    if (ditem) {
                        cacheItems[cont++] = ditem;
                        cacheItems[cont] = NULL;
            }
        }
                closedir(dir);

                // Sort the menu entries
                SortItems(cacheItems);

                // Add the menudir entries 
                for (cont=0; cacheItems[cont]; cont++) {
                    GnomeDesktopItem *ditem = cacheItems[cont];
                    const char *location   = gnome_desktop_item_get_location(ditem); 
                    if (!strncmp(location, "file://", 7)) location += 7;

                    const char *name = strrchr(location,'/');
                    name = name ? name + 1: location;
                    addEntry(dirname, name, strlen(dirname), ditem, false);
                }
            }
        }

        // Read all applications from the apps dir, and cache them (cacheAppItems)
        strcpy (dirname, APPLICATIONS_DIR);
        int cont=0;
        if (*cacheAppItems == NULL ) {
            for (cont=0; cacheItems[cont]; cont ++) {
                  cacheAppItems[cont] = cacheItems[cont];
            }
        DIR* dir = opendir(dirname);
        if (dir != 0) {
            struct dirent *file;

            while ((file = readdir(dir)) != NULL) {
                    if (*file->d_name != '.') {
                char fullpath[256];
                strcpy(fullpath, dirname);
                strcat(fullpath, file->d_name);
                GnomeDesktopItem *ditem =
                    gnome_desktop_item_new_from_file(fullpath,
                                                     (GnomeDesktopItemLoadFlags)0,
                                                     NULL);
                        cacheAppItems[cont++] = ditem;
                        cacheAppItems[cont] = NULL;
                        }
                    }
                closedir(dir);
                SortItems(cacheAppItems);
                }
            }

        // Search for the menu configuration for this .directory file
        MenuDir *md = NULL;
        for (int i=0; knownMenuDirs[i].filename; i++) {
             if (strstr(fPath, knownMenuDirs[i].filename)) {
                  md = &knownMenuDirs[i]; break;
            }
        }
        // Add all AppItems that match the configuration conditions
        strcpy (dirname, APPLICATIONS_DIR);
        for (cont=0; cacheAppItems[cont]; cont++) {
           GnomeDesktopItem *ditem = cacheAppItems[cont];
           const char *location   = gnome_desktop_item_get_location(ditem);
           const char *categories = gnome_desktop_item_get_string(ditem, GNOME_DESKTOP_ITEM_CATEGORIES);
           //const char *iname      = gnome_desktop_item_get_localestring(ditem, GNOME_DESKTOP_ITEM_NAME);
           bool ok = false;
	       if (categories && ( !md || md->includedCategories[0] || *md->excludedCategories[0])) {
               ok = true;
			   if (md) { // is a known menudir
                  for (int i=0; ok &&  md->includedCategories[i]; i++) {
                     if (!strstr(categories, md->includedCategories[i])) {
                         ok = false; break;
		             }
                  }
                  for (int i=0; ok &&  md->excludedCategories[i]; i++) {
                     if ( strstr(categories, md->excludedCategories[i]))  {
                         ok = false; break;
                     }
                  }
               } else { // is the root menu
                  for (int i=0; ok && root.includedCategories[i]; i++) {
                     if (!strstr(categories, root.includedCategories[i]))  {
                         ok = false; break;
                     }
                  }
                  for (int i=0; ok && root.excludedCategories[i]; i++) {
                     if ( strstr(categories, root.excludedCategories[i]))  {
                         ok = false; break;
                     }
                  }
               }
           }
           if (ok) {
               if (location && !strncmp(location, "file://", 7))
                   location += 7;
               if (!stat(location, &sb) && S_ISREG(sb.st_mode)) {
                    const char *name = strrchr(location,'/');
                    name = name ? name + 1: location;
                    addEntry(dirname, name, strlen(dirname), ditem, false);
               } 
           }
        }

        // Add submenus to this menu
        for (int i=0; md && md->subMenus[i]; i++) {
            addEntry(DIRECTORIES_DIR, md->subMenus[i], strlen(DIRECTORIES_DIR), NULL, false);
        }
    }
}

int makeMenu(const char *base_directory) {
    *cacheItems = NULL;
    *cacheAppItems = NULL;
    GnomeMenu *menu = new GnomeMenu();
    menu->populateMenu(base_directory);
    dumpMenu(menu,0);
    return 0;
}

int runFile(const char *dentry_path) {

    char arg[32];
    int i;

    GnomeDesktopItem *ditem =
        gnome_desktop_item_new_from_file(dentry_path,
                                         (GnomeDesktopItemLoadFlags)0,
                                         NULL);

    if (ditem == NULL) {
        return 1;
    } else {
//      FIXME: leads to segfault for some reason, so using execlp instead
//      gnome_desktop_item_launch(ditem, NULL, 0, NULL);

        const char *app = gnome_desktop_item_get_string(ditem, GNOME_DESKTOP_ITEM_EXEC);
        for (i = 0; app[i] && app[i] != ' '; ++i) {
            arg[i] = app[i];
        }
        arg[i] = '\0';

        execlp(arg, arg, NULL);
    }

    return 0;
}

int main(int argc, char **argv) {

    gtk_set_locale();
    gtk_init(&argc, &argv);

    gnome_vfs_init();

    for (char ** arg = argv + 1; arg < argv + argc; ++arg) {
        if (**arg == '-') {
            char *path = 0;
            if (IS_SWITCH("h", "help"))
                break;
            if ((path = GET_LONG_ARGUMENT("open")) != NULL) {
                return runFile(path);
            } else if ((path = GET_LONG_ARGUMENT("list")) != NULL) {
                return makeMenu(path);
            } else if (IS_SWITCH("e", "empty")) {
                showEmptyMenu=1;
            }
        }
    }
    msg("Usage: %s [ --open PATH | --list PATH ]", argv[0]);
}
#endif
